import { BaseEvent } from '#application/base/baseEvent'
import { UUID } from '#application/types'
import SpriteRepository from '#repositories/spriteRepository'
import { SpriteAction } from '#entities/spriteAction'
import sharp from 'sharp'

type Payload = {
  id: UUID
  name: string
  spriteActions: Array<{
    action: string
    sprites: string[]
    originX: number
    originY: number
    frameRate: number
  }>
}

export default class SpriteUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:sprite:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const spriteRepository = new SpriteRepository()
      const sprite = await spriteRepository.getById(data.id)
      if (!sprite) return callback(false)

      await spriteRepository.getEntityManager().populate(sprite, ['spriteActions'])

      // Update sprite in database
      await sprite.setName(data.name).save()

      const existingActions = sprite.getSpriteActions()

      // Remove all existing actions - we'll recreate the ones we need
      for (const existingAction of existingActions) {
        await spriteRepository.getEntityManager().removeAndFlush(existingAction)
      }

      // Create new actions
      for (const actionData of data.spriteActions) {
        const spriteAction = new SpriteAction()
        spriteAction.setSprite(sprite)
        sprite.getSpriteActions().add(spriteAction)

        spriteAction
          .setAction(actionData.action)
          .setSprites(actionData.sprites)
          .setOriginX(actionData.originX)
          .setOriginY(actionData.originY)
          .setFrameWidth(await this.calculateWidth(actionData.sprites[0]))
          .setFrameHeight(await this.calculateHeight(actionData.sprites[0]))
          .setFrameRate(actionData.frameRate)

        await spriteRepository.getEntityManager().persistAndFlush(spriteAction)
      }

      return callback(true)
    } catch (error) {
      console.error(`Error updating sprite ${data.id}:`, error)
      return callback(false)
    }
  }

  private async calculateWidth(base64: string): Promise<number> {
    const uri = base64.split(';base64,').pop()
    if (!uri) return 0

    const imgBuffer = Buffer.from(uri, 'base64')
    const image = await sharp(imgBuffer).metadata()
    return image.width ?? 0
  }

  private async calculateHeight(base64: string): Promise<number> {
    const uri = base64.split(';base64,').pop()
    if (!uri) return 0

    const imgBuffer = Buffer.from(uri, 'base64')
    const image = await sharp(imgBuffer).metadata()
    return image.height ?? 0
  }

  private generateSpriteSheet(sprites: string[]) {
    // In here comes a function that generates a sprite sheet from the given sprites using Sharp.
    // This function takes an array of base64 encoded sprites and generates a single png sprite sheet.
    // Then proceeds to save ths sprite sheet to the public/sprites/{spriteId}/ directory. The file name is {action}.png.
  }
}
